# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core.commands import CliCommandType
from ._client_factory import cf_nat_gateways


def load_command_table(self, _):
    nat_gateway_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.network.operations#NatGatewaysOperations.{}',
        client_factory=cf_nat_gateways)

    with self.command_group('network nat gateway', nat_gateway_sdk) as g:
        g.custom_command('create', 'create_nat_gateway', supports_no_wait=True)
        g.command('delete', 'begin_delete')
        g.custom_command('list', 'list_nat_gateway')
        g.show_command('show', 'get')
        g.generic_update_command('update', setter_name='begin_create_or_update', custom_func_name='update_nat_gateway')
        g.wait_command('wait')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.util import CLIError
from azure.core.exceptions import HttpResponseError
from azure.cli.core.azclierror import ValidationError

from azure.mgmt.media.models import (MediaService, MediaServiceIdentity, StorageAccount, CheckNameAvailabilityInput,
                                     SyncStorageKeysInput, ResourceIdentity, KeyDelivery,
                                     AccessControl)


def get_mediaservice(client, account_name, resource_group_name=None):
    return client.get(resource_group_name,
                      account_name) if resource_group_name else client.get_by_subscription(account_name)


def list_mediaservices(client, resource_group_name=None):
    return client.list(resource_group_name) if resource_group_name else client.list_by_subscription()


def create_mediaservice(client, resource_group_name, account_name, storage_account, location=None,
                        mi_system_assigned=False, mi_user_assigned=None, disable_public_network=False,
                        default_action=None, ip_allow_list=None, tags=None):
    storage_account_primary = StorageAccount(type='Primary', id=storage_account)
    return create_or_update_mediaservice(client, resource_group_name, account_name, [storage_account_primary],
                                         location, mi_system_assigned, mi_user_assigned, disable_public_network,
                                         default_action, ip_allow_list, tags)


def add_mediaservice_secondary_storage(client, resource_group_name, account_name, storage_account,
                                       system_assigned=False, user_assigned=None):
    ams = client.get(resource_group_name, account_name)
    if (system_assigned is False and user_assigned is None and ams.storage_authentication == 'ManagedIdentity'):
        error_msg = 'Please specify either system assigned identity or a user assigned identity'
        raise ValidationError(error_msg)

    storage_accounts_filtered = list(filter(lambda s: storage_account in s.id, ams.storage_accounts))

    storage_account_secondary = StorageAccount(type='Secondary', id=storage_account)
    if ams.storage_authentication == 'ManagedIdentity':
        storage_account_secondary.identity = ResourceIdentity(use_system_assigned_identity=system_assigned,
                                                              user_assigned_identity=user_assigned)
    if not storage_accounts_filtered:
        ams.storage_accounts.append(storage_account_secondary)

    media_service = MediaService(name=ams.name, location=ams.location, key_delivery=ams.key_delivery,
                                 identity=ams.identity, encryption=ams.encryption,
                                 storage_accounts=ams.storage_accounts,
                                 storage_authentication=ams.storage_authentication,
                                 public_network_access=ams.public_network_access)

    return client.create_or_update(resource_group_name, account_name, media_service)


def remove_mediaservice_secondary_storage(client, resource_group_name, account_name, storage_account):
    ams = client.get(resource_group_name, account_name)

    storage_accounts_filtered = list(filter(lambda s: storage_account not in s.id and 'Secondary' in s.type,
                                            ams.storage_accounts))

    primary_storage_account = list(filter(lambda s: 'Primary' in s.type, ams.storage_accounts))[0]
    storage_accounts_filtered.append(primary_storage_account)

    media_service = MediaService(name=ams.name, location=ams.location, key_delivery=ams.key_delivery,
                                 identity=ams.identity, encryption=ams.encryption,
                                 storage_accounts=storage_accounts_filtered,
                                 storage_authentication=ams.storage_authentication,
                                 public_network_access=ams.public_network_access)

    return client.create_or_update(resource_group_name, account_name, media_service)


def sync_storage_keys(client, resource_group_name, account_name, storage_account_id):
    parameters = SyncStorageKeysInput(id=storage_account_id)
    return client.sync_storage_keys(resource_group_name, account_name, parameters)


def set_mediaservice_trusted_storage(client, resource_group_name, account_name, storage_auth, storage_account_id=None,
                                     system_assigned=False, user_assigned=None):
    ams: MediaService = client.get(resource_group_name, account_name)\
        if resource_group_name else client.get_by_subscription(account_name)
    if storage_auth == 'ManagedIdentity' and storage_account_id is None:
        error_msg = 'Please specify a storage account id for the storage account whose identity you would like to set'
        raise ValidationError(error_msg)

    for storage_account in ams.storage_accounts:
        if storage_auth == 'ManagedIdentity':
            if storage_account.id.lower() == storage_account_id.lower():
                storage_account.identity = ResourceIdentity(use_system_assigned_identity=system_assigned,
                                                            user_assigned_identity=user_assigned)
        else:
            storage_account.identity = None

    media_service = MediaService(name=ams.name, location=ams.location, key_delivery=ams.key_delivery,
                                 identity=ams.identity, encryption=ams.encryption,
                                 storage_accounts=ams.storage_accounts, storage_authentication=storage_auth,
                                 public_network_access=ams.public_network_access)

    return client.create_or_update(resource_group_name, account_name, media_service)


def create_or_update_mediaservice(client, resource_group_name, account_name, storage_accounts=None,
                                  location=None, mi_system_assigned=False, mi_user_assigned=None,
                                  disable_public_network=False, default_action=None, ip_allow_list=None,
                                  tags=None):
    media_service = MediaService(location=location, storage_accounts=storage_accounts,
                                 tags=tags,
                                 public_network_access='Disabled' if disable_public_network else 'Enabled')

    if mi_system_assigned or mi_user_assigned:
        identity = MediaServiceIdentity(type='')
        identity.type = 'SystemAssigned' if mi_system_assigned else ''
        user_id_dict = {}
        if mi_user_assigned:
            identity.type = ','.join([identity.type, 'UserAssigned']) if identity.type else 'UserAssigned'
            for user_id in mi_user_assigned:
                user_id_dict[user_id] = {}
            identity.user_assigned_identities = user_id_dict
        media_service.identity = identity

    if default_action:
        media_service.key_delivery = KeyDelivery(access_control=AccessControl(default_action=default_action,
                                                                              ip_allow_list=ip_allow_list
                                                                              if default_action == 'Deny' else []))
    else:
        media_service.key_delivery = KeyDelivery(access_control=AccessControl(default_action="Allow",
                                                                              ip_allow_list=[]))

    return client.create_or_update(resource_group_name, account_name, media_service)


def mediaservice_update_getter(client, resource_group_name, account_name):

    try:
        return client.get(resource_group_name, account_name)
    except HttpResponseError as ex:
        raise CLIError(ex.message)


def update_mediaservice(instance, tags=None):
    if not instance:
        raise CLIError('The account resource was not found.')

    if tags:
        instance.tags = tags

    return instance


def check_name_availability(client, location, account_name):
    parameters = CheckNameAvailabilityInput(name=account_name, type='MICROSOFT.MEDIA/MEDIASERVICES')
    availability = client.check_name_availability(location_name=location, parameters=parameters)

    if availability.name_available:
        return 'Name available.'

    return availability.message



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=line-too-long
from azure.cli.core.commands import CliCommandType
from azure.cli.command_modules.privatedns._client_factory import (cf_privatedns_mgmt_zones, cf_privatedns_mgmt_virtual_network_links, cf_privatedns_mgmt_record_sets)
from azure.cli.command_modules.privatedns._format import (transform_privatedns_zone_table_output, transform_privatedns_link_table_output, transform_privatedns_record_set_output, transform_privatedns_record_set_table_output)


def load_command_table(self, _):

    network_privatedns_zone_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.privatedns.operations#PrivateZonesOperations.{}',
        client_factory=cf_privatedns_mgmt_zones
    )

    network_privatedns_virtual_network_link_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.privatedns.operations#VirtualNetworkLinksOperations.{}',
        client_factory=cf_privatedns_mgmt_virtual_network_links
    )

    network_privatedns_record_set_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.privatedns.operations#RecordSetsOperations.{}',
        client_factory=cf_privatedns_mgmt_record_sets
    )

    network_privatedns_custom = CliCommandType(
        operations_tmpl='azure.cli.command_modules.privatedns.custom#{}',
        client_factory=cf_privatedns_mgmt_record_sets
    )

    with self.command_group('network private-dns zone', network_privatedns_zone_sdk) as g:
        g.command('delete', 'begin_delete', confirmation=True, supports_no_wait=True)
        g.show_command('show', 'get', table_transformer=transform_privatedns_zone_table_output)
        g.custom_command('list', 'list_privatedns_zones', client_factory=cf_privatedns_mgmt_zones, table_transformer=transform_privatedns_zone_table_output)
        g.custom_command('import', 'import_zone')
        g.custom_command('export', 'export_zone')
        g.custom_command('create', 'create_privatedns_zone', client_factory=cf_privatedns_mgmt_zones, supports_no_wait=True)
        g.generic_update_command('update', setter_name='begin_update', custom_func_name='update_privatedns_zone', supports_no_wait=True)
        g.wait_command('wait')

    with self.command_group('network private-dns link vnet', network_privatedns_virtual_network_link_sdk) as g:
        g.command('delete', 'begin_delete', confirmation=True, supports_no_wait=True)
        g.show_command('show', 'get', table_transformer=transform_privatedns_link_table_output)
        g.command('list', 'list', table_transformer=transform_privatedns_link_table_output)
        g.custom_command('create', 'create_privatedns_link', client_factory=cf_privatedns_mgmt_virtual_network_links, supports_no_wait=True)
        g.generic_update_command('update', setter_name='update_privatedns_link', setter_type=network_privatedns_custom, supports_no_wait=True)
        g.wait_command('wait')

    with self.command_group('network private-dns record-set') as g:
        g.custom_command('list', 'list_privatedns_record_set', client_factory=cf_privatedns_mgmt_record_sets, transform=transform_privatedns_record_set_output, table_transformer=transform_privatedns_record_set_table_output)

    supported_records = ['a', 'aaaa', 'mx', 'ptr', 'srv', 'txt']
    for record in supported_records:
        with self.command_group('network private-dns record-set {}'.format(record), network_privatedns_record_set_sdk) as g:
            g.show_command('show', 'get', transform=transform_privatedns_record_set_output, table_transformer=transform_privatedns_record_set_table_output)
            g.command('delete', 'delete', confirmation=True)
            g.custom_command('list', 'list_privatedns_record_set', client_factory=cf_privatedns_mgmt_record_sets, transform=transform_privatedns_record_set_output, table_transformer=transform_privatedns_record_set_table_output)
            g.custom_command('create', 'create_privatedns_record_set', client_factory=cf_privatedns_mgmt_record_sets, transform=transform_privatedns_record_set_output)
            g.custom_command('add-record', 'add_privatedns_{}_record'.format(record), client_factory=cf_privatedns_mgmt_record_sets, transform=transform_privatedns_record_set_output)
            g.custom_command('remove-record', 'remove_privatedns_{}_record'.format(record), client_factory=cf_privatedns_mgmt_record_sets, transform=transform_privatedns_record_set_output)
            g.generic_update_command('update', setter_name='update', custom_func_name='update_privatedns_record_set', transform=transform_privatedns_record_set_output)

    with self.command_group('network private-dns record-set soa', network_privatedns_record_set_sdk) as g:
        g.show_command('show', 'get', transform=transform_privatedns_record_set_output, table_transformer=transform_privatedns_record_set_table_output)
        g.custom_command('update', 'update_privatedns_soa_record', client_factory=cf_privatedns_mgmt_record_sets, transform=transform_privatedns_record_set_output)

    with self.command_group('network private-dns record-set cname', network_privatedns_record_set_sdk) as g:
        g.show_command('show', 'get', transform=transform_privatedns_record_set_output, table_transformer=transform_privatedns_record_set_table_output)
        g.command('delete', 'delete', confirmation=True)
        g.custom_command('list', 'list_privatedns_record_set', client_factory=cf_privatedns_mgmt_record_sets, transform=transform_privatedns_record_set_output, table_transformer=transform_privatedns_record_set_table_output)
        g.custom_command('create', 'create_privatedns_record_set', client_factory=cf_privatedns_mgmt_record_sets, transform=transform_privatedns_record_set_output)
        g.custom_command('set-record', 'add_privatedns_cname_record', client_factory=cf_privatedns_mgmt_record_sets, transform=transform_privatedns_record_set_output)
        g.custom_command('remove-record', 'remove_privatedns_cname_record', client_factory=cf_privatedns_mgmt_record_sets, transform=transform_privatedns_record_set_output)
        g.generic_update_command('update', setter_name='update', custom_func_name='update_privatedns_record_set', transform=transform_privatedns_record_set_output)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


def cf_dls_account(cli_ctx, _):
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    from azure.mgmt.datalake.store import DataLakeStoreAccountManagementClient
    return get_mgmt_service_client(cli_ctx, DataLakeStoreAccountManagementClient).accounts


def cf_dls_account_firewall(cli_ctx, _):
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    from azure.mgmt.datalake.store import DataLakeStoreAccountManagementClient
    return get_mgmt_service_client(cli_ctx, DataLakeStoreAccountManagementClient).firewall_rules


def cf_dls_account_virtual_network(cli_ctx, _):
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    from azure.mgmt.datalake.store import DataLakeStoreAccountManagementClient
    return get_mgmt_service_client(cli_ctx, DataLakeStoreAccountManagementClient).virtual_network_rules


def cf_dls_account_trusted_provider(cli_ctx, _):
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    from azure.mgmt.datalake.store import DataLakeStoreAccountManagementClient
    return get_mgmt_service_client(cli_ctx, DataLakeStoreAccountManagementClient).trusted_id_providers


def cf_dls_filesystem(cli_ctx, account_name):
    from azure.datalake.store import core
    from azure.cli.core._profile import Profile

    profile = Profile(cli_ctx=cli_ctx)
    subscription_id = None
    cred, subscription_id, _ = profile.get_login_credentials(
        subscription_id=subscription_id,
        resource=cli_ctx.cloud.endpoints.active_directory_data_lake_resource_id)
    return core.AzureDLFileSystem(
        token=cred,
        store_name=account_name,
        url_suffix=cli_ctx.cloud.suffixes.azure_datalake_store_file_system_endpoint)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=line-too-long,too-many-lines,too-many-branches,too-many-locals

import time

from azure.core.exceptions import HttpResponseError
from azure.cli.core.azclierror import InvalidArgumentValueError
from azure.cli.core.commands import LongRunningOperation
from azure.mgmt.servicefabricmanagedclusters.models import (ApplicationTypeResource,
                                                            ApplicationTypeVersionResource,
                                                            ApplicationResource,
                                                            ApplicationUpgradePolicy,
                                                            ApplicationHealthPolicy,
                                                            NamedPartitionScheme,
                                                            PartitionScheme,
                                                            RollingUpgradeMonitoringPolicy,
                                                            ServiceCorrelation,
                                                            ServiceLoadMetric,
                                                            ServiceResource,
                                                            StatefulServiceProperties,
                                                            StatelessServiceProperties,
                                                            ServiceTypeHealthPolicy,
                                                            ServiceKind,
                                                            SingletonPartitionScheme,
                                                            UniformInt64RangePartitionScheme)
from azure.cli.command_modules.servicefabric._sf_utils import (add_to_collection,
                                                               _get_managed_cluster_location,
                                                               delete_from_collection,
                                                               find_in_collection,
                                                               update_in_collection)
from knack.log import get_logger

logger = get_logger(__name__)

# Constants
APP_VERSION_ARM_RESOURCE_ID_FORMAT = ("/subscriptions/{subscription}/resourceGroups/{rg}/providers/"
                                      "Microsoft.ServiceFabric/managedclusters/{cluster}/"
                                      "applicationTypes/{appType}/versions/{version}")
CONSIDER_WARNING_AS_ERROR_DEFAULT = False
MAX_PERCENT_UNHEALTHY_DEPLOYED_APPLICATIONS_DEFAULT = 0
FAILURE_ACTION_DEFAULT = "Manual"
HEALTH_CHECK_STABLE_DURATION_DEFAULT = time.strftime('%H:%M:%S', time.gmtime(120))
HEALTH_CHECK_RETRY_TIMEOUT_DEFAULT = time.strftime('%H:%M:%S', time.gmtime(600))
HEALTH_CHECK_WAIT_DURATION_DEFAULT = time.strftime('%H:%M:%S', time.gmtime(0))
_twelve_hours = 43200
UPGRADE_TIMEOUT_DEFAULT = time.strftime('%H:%M:%S', time.gmtime(_twelve_hours))
UPGRADE_DOMAIN_TIMEOUT = time.strftime('%H:%M:%S', time.gmtime(_twelve_hours))
SERVICE_TYPE_UNHEALTHY_SERVICES_MAX_PERCENT_DEFAULT = 0
SERVICE_TYPE_MAX_PERCENT_UNHEALTHY_REPLICAS_PER_PARTITION_DEFAULT = 0
SERVICE_TYPE_MAX_PERCENT_UNHEALTHY_PARTITIONS_PER_SERVICE_DEFAULT = 0


def create_app(cmd,
               client,
               resource_group_name,
               cluster_name,
               application_type_name,
               application_type_version,
               application_name,
               package_url=None,
               application_parameters=None,
               tags=None):
    location = _get_managed_cluster_location(cmd.cli_ctx, resource_group_name, cluster_name)
    if package_url is not None:
        create_app_type_version(cmd, client, resource_group_name, cluster_name, application_type_name, application_type_version, package_url)

    try:
        apps = client.applications.list(resource_group_name, cluster_name)
        for app in apps:
            if app.name.lower() == application_name.lower():
                logger.info("Application '%s' already exists", application_name)
                return app

        new_app_type_version = _format_app_version(cmd.cli_ctx, resource_group_name, cluster_name, application_type_name, application_type_version)
        appResource = ApplicationResource(version=new_app_type_version,
                                          parameters=application_parameters,
                                          location=location,
                                          tags=tags)
        appResource.name = application_name
        poller = client.applications.begin_create_or_update(resource_group_name, cluster_name, application_name, appResource)
        return LongRunningOperation(cmd.cli_ctx)(poller)
    except HttpResponseError as ex:
        logger.error("HttpResponseError: %s", ex)
        raise


def update_app(cmd,
               client,
               resource_group_name,
               cluster_name,
               application_name,
               application_type_version=None,
               application_parameters=None,
               force_restart=False,
               recreate_application=False,
               upgrade_replica_set_check_timeout=None,
               instance_close_delay_duration=None,
               failure_action=None,
               upgrade_mode=None,
               health_check_retry_timeout=None,
               health_check_wait_duration=None,
               health_check_stable_duration=None,
               upgrade_domain_timeout=None,
               upgrade_timeout=None,
               consider_warning_as_error=False,
               default_service_type_max_percent_unhealthy_partitions_per_service=None,
               default_service_type_max_percent_unhealthy_replicas_per_partition=None,
               default_service_type_max_percent_unhealthy_services=None,
               max_percent_unhealthy_deployed_applications=None,
               service_type_health_policy_map=None,
               tags=None):
    try:
        currentApp = client.applications.get(resource_group_name, cluster_name, application_name)
        appResource = currentApp
        # TODO: change to patch once is supported in the rp
        # appResourceUpdate: ApplicationResourceUpdate = ApplicationResourceUpdate()

        if application_type_version:
            appResource.version = _replace_app_version(appResource.version, application_type_version)
        if application_parameters:
            appResource.parameters.update(application_parameters)
        if tags:
            appResource.tags = tags

        appResource.upgrade_policy = _set_upgrade_policy(currentApp.upgrade_policy,
                                                         force_restart,
                                                         recreate_application,
                                                         upgrade_replica_set_check_timeout,
                                                         instance_close_delay_duration,
                                                         failure_action,
                                                         upgrade_mode,
                                                         health_check_retry_timeout,
                                                         health_check_wait_duration,
                                                         health_check_stable_duration,
                                                         upgrade_domain_timeout,
                                                         upgrade_timeout,
                                                         consider_warning_as_error,
                                                         default_service_type_max_percent_unhealthy_partitions_per_service,
                                                         default_service_type_max_percent_unhealthy_replicas_per_partition,
                                                         default_service_type_max_percent_unhealthy_services,
                                                         max_percent_unhealthy_deployed_applications,
                                                         service_type_health_policy_map)

        # TODO: change to patch once the fix is supported in the rp
        # client.applications.update(resource_group_name, cluster_name, application_name, appResourceUpdate)
        poller = client.applications.begin_create_or_update(resource_group_name, cluster_name, application_name, appResource)
        return LongRunningOperation(cmd.cli_ctx)(poller)
    except HttpResponseError as ex:
        logger.error("HttpResponseError: %s", ex)
        raise


def create_app_type(cmd, client, resource_group_name, cluster_name, application_type_name, tags=None):
    try:
        location = _get_managed_cluster_location(cmd.cli_ctx, resource_group_name, cluster_name)
        appTypes = client.application_types.list(resource_group_name, cluster_name)
        for appType in appTypes:
            if appType.name.lower() == application_type_name.lower():
                logger.info("Application type '%s' already exists", application_type_name)
                return appType

        appTypeResource = ApplicationTypeResource(location=location, tags=tags)
        logger.info("Creating application type '%s'", application_type_name)
        return client.application_types.create_or_update(
            resource_group_name,
            cluster_name,
            application_type_name,
            appTypeResource)
    except HttpResponseError as ex:
        logger.error("HttpResponseError: %s", ex)
        raise


def update_app_type(client, resource_group_name, cluster_name, application_type_name, tags=None):
    try:
        currentAppType = client.application_types.get(resource_group_name, cluster_name, application_type_name)

        if tags:
            currentAppType.tags = tags
        logger.info("Updating application type '%s'", application_type_name)
        return client.application_types.create_or_update(
            resource_group_name,
            cluster_name,
            application_type_name,
            currentAppType)
    except HttpResponseError as ex:
        logger.error("HttpResponseError: %s", ex)
        raise


def create_app_type_version(cmd,
                            client,
                            resource_group_name,
                            cluster_name,
                            application_type_name,
                            version,
                            package_url,
                            tags=None):
    location = _get_managed_cluster_location(cmd.cli_ctx, resource_group_name, cluster_name)
    create_app_type(cmd, client, resource_group_name, cluster_name, application_type_name)
    try:
        appTypeVerions = client.application_type_versions.list_by_application_types(resource_group_name, cluster_name, application_type_name)
        for appTypeVerion in appTypeVerions:
            if appTypeVerion.name.lower() == version.lower():
                logger.error("Application type version '%s' already exists", version)
                return appTypeVerion

        appTypeVersionResource = ApplicationTypeVersionResource(app_package_url=package_url, location=location, tags=tags)
        logger.info("Creating application type version %s:%s", application_type_name, version)
        poller = client.application_type_versions.begin_create_or_update(resource_group_name,
                                                                         cluster_name,
                                                                         application_type_name,
                                                                         version,
                                                                         appTypeVersionResource)
        return LongRunningOperation(cmd.cli_ctx)(poller)
    except HttpResponseError as ex:
        logger.error("HttpResponseError: %s", ex)
        raise


def update_app_type_version(client,
                            resource_group_name,
                            cluster_name,
                            application_type_name,
                            version,
                            package_url=None,
                            tags=None):
    try:
        currentAppTypeVersion = client.application_type_versions.get(
            resource_group_name,
            cluster_name,
            application_type_name,
            version)

        if package_url is not None:
            currentAppTypeVersion.app_package_url = package_url

        if tags is not None:
            currentAppTypeVersion.tags = tags

        logger.info("Updating application type version %s:%s", application_type_name, version)
        return client.application_type_versions.begin_create_or_update(resource_group_name,
                                                                       cluster_name,
                                                                       application_type_name,
                                                                       version,
                                                                       currentAppTypeVersion).result()
    except HttpResponseError as ex:
        logger.error("HttpResponseError: %s", ex)
        raise


def create_service(cmd,
                   client,
                   resource_group_name,
                   cluster_name,
                   application_name,
                   service_name,
                   service_type,
                   state,
                   default_move_cost=None,
                   placement_constraints=None,
                   service_package_activation_mode=None,
                   target_replica_set_size=None,
                   min_replica_set_size=None,
                   has_persisted_state=None,
                   service_placement_time_limit=None,
                   stand_by_replica_keep_duration=None,
                   quorum_loss_wait_duration=None,
                   replica_restart_wait_duration=None,
                   instance_count=None,
                   min_instance_count=None,
                   min_instance_percentage=None,
                   partition_scheme='singleton',
                   partition_count=None,
                   low_key=None,
                   high_key=None,
                   partition_names=None,
                   tags=None):
    try:
        location = _get_managed_cluster_location(cmd.cli_ctx, resource_group_name, cluster_name)
        services = client.services.list_by_applications(resource_group_name, cluster_name, application_name)
        for service in services:
            if service.name.lower() == service_name.lower():
                logger.error("Service '%s' already exists", service_name)
                return service

        serviceResource = ServiceResource(location=location, tags=tags)
        serviceResource.name = service_name

        if state.lower() == ServiceKind.STATELESS.lower():
            properties = StatelessServiceProperties(
                service_type_name=service_type,
                instance_count=instance_count,
                partition_description=_set_partition_description(partition_scheme, partition_names, partition_count, low_key, high_key)
            )
            serviceResource.properties = _set_stateless_service_properties(properties,
                                                                           min_instance_count,
                                                                           min_instance_percentage)

        elif state.lower() == ServiceKind.STATEFUL.lower():
            properties = StatefulServiceProperties(
                service_type_name=service_type,
                instance_count=instance_count,
                partition_description=_set_partition_description(partition_scheme, partition_names, partition_count, low_key, high_key),
                min_replica_set_size=min_replica_set_size,
                target_replica_set_size=target_replica_set_size
            )
            serviceResource.properties = _set_stateful_service_properties(properties,
                                                                          has_persisted_state,
                                                                          service_placement_time_limit,
                                                                          stand_by_replica_keep_duration,
                                                                          quorum_loss_wait_duration,
                                                                          replica_restart_wait_duration)
        else:
            raise InvalidArgumentValueError("Invalid --state '%s': service state is not valid." % state)

        serviceResource.properties.service_load_metrics = []
        serviceResource.properties.correlation_scheme = []

        if default_move_cost is not None:
            serviceResource.properties.default_move_cost = default_move_cost
        if placement_constraints is not None:
            serviceResource.properties.placement_constraints = placement_constraints
        if service_package_activation_mode is not None:
            serviceResource.properties.service_package_activation_mode = service_package_activation_mode

        poller = client.services.begin_create_or_update(resource_group_name, cluster_name, application_name, service_name, serviceResource)
        return LongRunningOperation(cmd.cli_ctx)(poller)
    except HttpResponseError as ex:
        logger.error("HttpResponseError: %s", ex)
        raise


def update_service(cmd,
                   client,
                   resource_group_name,
                   cluster_name,
                   application_name,
                   service_name,
                   default_move_cost=None,
                   placement_constraints=None,
                   target_replica_set_size=None,
                   min_replica_set_size=None,
                   service_placement_time_limit=None,
                   stand_by_replica_keep_duration=None,
                   quorum_loss_wait_duration=None,
                   replica_restart_wait_duration=None,
                   instance_count=None,
                   min_instance_count=None,
                   min_instance_percentage=None,
                   tags=None):
    try:
        currentService = client.services.get(resource_group_name, cluster_name, application_name, service_name)

        state = currentService.properties.service_kind
        logger.info("Updating service '%s'", service_name)

        if tags:
            currentService.tags = tags

        if state.lower() == ServiceKind.STATELESS.lower():
            if instance_count is not None:
                currentService.properties.instance_count = instance_count
            currentService.properties = _set_stateless_service_properties(currentService.properties,
                                                                          min_instance_count,
                                                                          min_instance_percentage)
        elif state.lower() == ServiceKind.STATEFUL.lower():
            if min_replica_set_size is not None:
                currentService.properties.min_replica_set_size = min_replica_set_size
            if target_replica_set_size is not None:
                currentService.properties.target_replica_set_size = target_replica_set_size
            currentService.properties = _set_stateful_service_properties(currentService.properties,
                                                                         None,
                                                                         service_placement_time_limit,
                                                                         stand_by_replica_keep_duration,
                                                                         quorum_loss_wait_duration,
                                                                         replica_restart_wait_duration)
        else:
            raise InvalidArgumentValueError("Invalid --state '%s': service state is not valid." % state)

        if default_move_cost is not None:
            currentService.properties.default_move_cost = default_move_cost
        if placement_constraints is not None:
            currentService.properties.placement_constraints = placement_constraints

        poller = client.services.begin_create_or_update(resource_group_name, cluster_name, application_name, service_name, currentService)
        return LongRunningOperation(cmd.cli_ctx)(poller)
    except HttpResponseError as ex:
        logger.error("HttpResponseError: %s", ex)
        raise


def create_service_load_metric(cmd,
                               client,
                               resource_group_name,
                               cluster_name,
                               application_name,
                               service_name,
                               metric_name,
                               weight=None,
                               primary_default_load=None,
                               secondary_default_load=None,
                               default_load=None):

    service = client.services.get(resource_group_name, cluster_name, application_name, service_name)
    new_metric = ServiceLoadMetric(
        name=metric_name,
        weight=weight,
        primary_default_load=primary_default_load,
        secondary_default_load=secondary_default_load,
        default_load=default_load
    )
    # add the new child to the parent collection
    add_to_collection(service.properties, 'service_load_metrics', new_metric, 'name')
    # update the parent object
    poller = client.services.begin_create_or_update(resource_group_name, cluster_name, application_name, service_name, service)
    return LongRunningOperation(cmd.cli_ctx)(poller)


def update_service_load_metric(cmd,
                               client,
                               resource_group_name,
                               cluster_name,
                               application_name,
                               service_name,
                               metric_name,
                               weight=None,
                               primary_default_load=None,
                               secondary_default_load=None,
                               default_load=None):

    service = client.services.get(resource_group_name, cluster_name, application_name, service_name)
    existing_metric = find_in_collection(service.properties, 'service_load_metrics', 'name', metric_name)
    if existing_metric is None:
        logger.error('Metric %s does not exist.', metric_name)
        return None

    updated_metric = ServiceLoadMetric(
        name=metric_name,
        weight=weight if weight is not None else existing_metric.weight,
        primary_default_load=primary_default_load if primary_default_load is not None else existing_metric.primary_default_load,
        secondary_default_load=secondary_default_load if secondary_default_load is not None else existing_metric.secondary_default_load,
        default_load=default_load if default_load is not None else existing_metric.default_load,
    )
    # add the new child to the parent collection
    update_in_collection(service.properties, 'service_load_metrics', updated_metric, 'name')
    # update the parent object
    poller = client.services.begin_create_or_update(resource_group_name, cluster_name, application_name, service_name, service)
    return LongRunningOperation(cmd.cli_ctx)(poller)


def delete_service_load_metric(cmd,
                               client,
                               resource_group_name,
                               cluster_name,
                               application_name,
                               service_name,
                               metric_name):

    service = client.services.get(resource_group_name, cluster_name, application_name, service_name)

    # add the new child to the parent collection
    delete_from_collection(service.properties, 'service_load_metrics', 'name', metric_name)
    # update the parent object
    poller = client.services.begin_create_or_update(resource_group_name, cluster_name, application_name, service_name, service)
    return LongRunningOperation(cmd.cli_ctx)(poller)


def create_service_correlation(cmd,
                               client,
                               resource_group_name,
                               cluster_name,
                               application_name,
                               service_name,
                               scheme,
                               correlated_service_name):

    service = client.services.get(resource_group_name, cluster_name, application_name, service_name)
    new_correlation = ServiceCorrelation(
        service_name=correlated_service_name,
        scheme=scheme
    )
    # add the new child to the parent collection
    add_to_collection(service.properties, 'correlation_scheme', new_correlation, 'service_name')
    # update the parent object
    poller = client.services.begin_create_or_update(resource_group_name, cluster_name, application_name, service_name, service)
    return LongRunningOperation(cmd.cli_ctx)(poller)


def update_service_correlation(cmd,
                               client,
                               resource_group_name,
                               cluster_name,
                               application_name,
                               service_name,
                               scheme,
                               correlated_service_name):

    service = client.services.get(resource_group_name, cluster_name, application_name, service_name)
    existing_correlation = find_in_collection(service.properties, 'correlation_scheme', 'service_name', correlated_service_name)
    if existing_correlation is None:
        logger.error('Correlation %s does not exist.', correlated_service_name)
        return None

    updated_correlation = ServiceCorrelation(
        service_name=correlated_service_name,
        scheme=scheme
    )
    # add the new child to the parent collection
    update_in_collection(service.properties, 'correlation_scheme', updated_correlation, 'service_name')
    # update the parent object
    poller = client.services.begin_create_or_update(resource_group_name, cluster_name, application_name, service_name, service)
    return LongRunningOperation(cmd.cli_ctx)(poller)


def delete_service_correlation(cmd,
                               client,
                               resource_group_name,
                               cluster_name,
                               application_name,
                               service_name,
                               correlated_service_name):

    service = client.services.get(resource_group_name, cluster_name, application_name, service_name)

    # add the new child to the parent collection
    delete_from_collection(service.properties, 'correlation_scheme', 'service_name', correlated_service_name)
    # update the parent object
    poller = client.services.begin_create_or_update(resource_group_name, cluster_name, application_name, service_name, service)
    return LongRunningOperation(cmd.cli_ctx)(poller)


# Managed application helpers
def _format_app_version(cli_ctx, rg_name, cluster_name, app_type, new_app_version):
    from azure.cli.core.commands.client_factory import get_subscription_id
    subscription_id = get_subscription_id(cli_ctx)
    return APP_VERSION_ARM_RESOURCE_ID_FORMAT.format(
        subscription=subscription_id,
        rg=rg_name,
        cluster=cluster_name,
        appType=app_type,
        version=new_app_version
    )


def _replace_app_version(old_app_version, new_app_version):
    split_version_id = old_app_version.rsplit(sep='/', maxsplit=1)
    split_version_id[-1] = new_app_version
    return "/".join(split_version_id)


def _set_service_type_health_policy(service_type_health_policy_map):
    def _policy_string_to_object(value_string):
        try:
            max_percent_unhealthy_partitions_per_service, max_percent_unhealthy_replicas_per_partition, max_percent_unhealthy_services = map(int, value_string.split(","))
        except:
            raise InvalidArgumentValueError("Invalid --service-type-health-policy-map '%s': service type health policy map is not valid." % service_type_health_policy_map)
        return ServiceTypeHealthPolicy(
            max_percent_unhealthy_partitions_per_service=max_percent_unhealthy_partitions_per_service,
            max_percent_unhealthy_replicas_per_partition=max_percent_unhealthy_replicas_per_partition,
            max_percent_unhealthy_services=max_percent_unhealthy_services
        )
    return {k: _policy_string_to_object(v) for k, v in service_type_health_policy_map.items()}


def _set_upgrade_policy(current_upgrade_policy,
                        force_restart,
                        recreate_application,
                        upgrade_replica_set_check_timeout,
                        instance_close_delay_duration,
                        failure_action,
                        upgrade_mode,
                        health_check_retry_timeout,
                        health_check_wait_duration,
                        health_check_stable_duration,
                        upgrade_domain_timeout,
                        upgrade_timeout,
                        consider_warning_as_error,
                        default_service_type_max_percent_unhealthy_partitions_per_service,
                        default_service_type_max_percent_unhealthy_replicas_per_partition,
                        default_service_type_max_percent_unhealthy_services,
                        max_percent_unhealthy_deployed_applications,
                        service_type_health_policy_map):
    if current_upgrade_policy is None:
        current_upgrade_policy = ApplicationUpgradePolicy()

    if force_restart is not None:
        current_upgrade_policy.force_restart = force_restart
    if recreate_application is not None:
        current_upgrade_policy.recreate_application = recreate_application
    if upgrade_mode:
        current_upgrade_policy.upgrade_mode = upgrade_mode
    if upgrade_replica_set_check_timeout is not None:
        current_upgrade_policy.upgrade_replica_set_check_timeout = upgrade_replica_set_check_timeout
    if instance_close_delay_duration is not None:
        current_upgrade_policy.instance_close_delay_duration = instance_close_delay_duration

    # RollingUpgradeMonitoringPolicy
    if current_upgrade_policy.rolling_upgrade_monitoring_policy is None:
        # initialize with defaults
        current_upgrade_policy.rolling_upgrade_monitoring_policy = RollingUpgradeMonitoringPolicy(failure_action=FAILURE_ACTION_DEFAULT,
                                                                                                  health_check_stable_duration=HEALTH_CHECK_STABLE_DURATION_DEFAULT,
                                                                                                  health_check_retry_timeout=HEALTH_CHECK_RETRY_TIMEOUT_DEFAULT,
                                                                                                  health_check_wait_duration=HEALTH_CHECK_WAIT_DURATION_DEFAULT,
                                                                                                  upgrade_timeout=UPGRADE_TIMEOUT_DEFAULT,
                                                                                                  upgrade_domain_timeout=UPGRADE_DOMAIN_TIMEOUT)

    if failure_action:
        current_upgrade_policy.rolling_upgrade_monitoring_policy.failure_action = failure_action
    if health_check_stable_duration is not None:
        current_upgrade_policy.rolling_upgrade_monitoring_policy.health_check_stable_duration = time.strftime('%H:%M:%S', time.gmtime(health_check_stable_duration))
    if health_check_retry_timeout is not None:
        current_upgrade_policy.rolling_upgrade_monitoring_policy.health_check_retry_timeout = time.strftime('%H:%M:%S', time.gmtime(health_check_retry_timeout))
    if health_check_wait_duration is not None:
        current_upgrade_policy.rolling_upgrade_monitoring_policy.health_check_wait_duration = time.strftime('%H:%M:%S', time.gmtime(health_check_wait_duration))
    if upgrade_timeout is not None:
        current_upgrade_policy.rolling_upgrade_monitoring_policy.upgrade_timeout = time.strftime('%H:%M:%S', time.gmtime(upgrade_timeout))
    if upgrade_domain_timeout is not None:
        current_upgrade_policy.rolling_upgrade_monitoring_policy.upgrade_domain_timeout = time.strftime('%H:%M:%S', time.gmtime(upgrade_domain_timeout))

    # ApplicationHealthPolicy
    if current_upgrade_policy.application_health_policy is None:
        current_upgrade_policy.application_health_policy = ApplicationHealthPolicy(
            consider_warning_as_error=CONSIDER_WARNING_AS_ERROR_DEFAULT,
            max_percent_unhealthy_deployed_applications=MAX_PERCENT_UNHEALTHY_DEPLOYED_APPLICATIONS_DEFAULT
        )

    if consider_warning_as_error:
        current_upgrade_policy.application_health_policy.consider_warning_as_error = True

    if current_upgrade_policy.application_health_policy.default_service_type_health_policy is None:
        current_upgrade_policy.application_health_policy.default_service_type_health_policy = ServiceTypeHealthPolicy(
            max_percent_unhealthy_partitions_per_service=SERVICE_TYPE_MAX_PERCENT_UNHEALTHY_PARTITIONS_PER_SERVICE_DEFAULT,
            max_percent_unhealthy_replicas_per_partition=SERVICE_TYPE_MAX_PERCENT_UNHEALTHY_REPLICAS_PER_PARTITION_DEFAULT,
            max_percent_unhealthy_services=SERVICE_TYPE_UNHEALTHY_SERVICES_MAX_PERCENT_DEFAULT)

    if default_service_type_max_percent_unhealthy_partitions_per_service is not None:
        current_upgrade_policy.application_health_policy.default_service_type_health_policy.max_percent_unhealthy_partitions_per_service \
            = default_service_type_max_percent_unhealthy_partitions_per_service
    if default_service_type_max_percent_unhealthy_replicas_per_partition is not None:
        current_upgrade_policy.application_health_policy.default_service_type_health_policy.max_percent_unhealthy_replicas_per_partition \
            = default_service_type_max_percent_unhealthy_replicas_per_partition
    if default_service_type_max_percent_unhealthy_services is not None:
        current_upgrade_policy.application_health_policy.default_service_type_health_policy.max_percent_unhealthy_partitions_per_service \
            = default_service_type_max_percent_unhealthy_services

    if max_percent_unhealthy_deployed_applications is not None:
        current_upgrade_policy.application_health_policy.max_percent_unhealthy_deployed_applications \
            = max_percent_unhealthy_deployed_applications

    if service_type_health_policy_map:
        current_upgrade_policy.application_health_policy.service_type_health_policy_map = _set_service_type_health_policy(service_type_health_policy_map)
    return current_upgrade_policy


def _set_partition_description(partition_scheme, partition_names, partition_count, low_key, high_key):
    partition_description = None
    if partition_scheme.lower() == PartitionScheme.SINGLETON.lower():
        partition_description = SingletonPartitionScheme()
    elif partition_scheme.lower() == PartitionScheme.NAMED.lower():
        partition_description = NamedPartitionScheme(
            names=partition_names
        )
    elif partition_scheme.lower() == PartitionScheme.UNIFORM_INT64_RANGE.lower():
        partition_description = UniformInt64RangePartitionScheme(
            count=partition_count,
            low_key=low_key,
            high_key=high_key
        )
    else:
        raise InvalidArgumentValueError("Invalid --partition-scheme '%s': service partition scheme is not valid." % partition_scheme)
    return partition_description


def _set_stateless_service_properties(properties, min_instance_count, min_instance_percentage):
    # Optional
    if min_instance_count is not None:
        properties.min_instance_count = min_instance_count
    if min_instance_percentage is not None:
        properties.min_instance_percentage = min_instance_percentage
    return properties


def _set_stateful_service_properties(properties,
                                     has_persisted_state,
                                     service_placement_time_limit,
                                     stand_by_replica_keep_duration,
                                     quorum_loss_wait_duration,
                                     replica_restart_wait_duration):
    # Optional
    if has_persisted_state is not None:
        properties.has_persisted_state = has_persisted_state
    if service_placement_time_limit is not None:
        properties.service_placement_time_limit = service_placement_time_limit
    if stand_by_replica_keep_duration is not None:
        properties.stand_by_replica_keep_duration = stand_by_replica_keep_duration
    if quorum_loss_wait_duration is not None:
        properties.quorum_loss_wait_duration = quorum_loss_wait_duration
    if replica_restart_wait_duration is not None:
        properties.replica_restart_wait_duration = replica_restart_wait_duration
    return properties



# coding=utf-8
# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.help_files import helps  # pylint: disable=unused-import
# pylint: disable=line-too-long, too-many-lines

helps['kusto'] = """
type: group
short-summary: Manage Azure Kusto resources
long-summary: This module will not be supported starting January 1, 2021. Please execute "az extension add -n kusto" to install the supported Kusto extension
"""

helps['kusto cluster'] = """
type: group
short-summary: Manage Azure Kusto clusters.
long-summary: This module will not be supported starting January 1, 2021. Please execute "az extension add -n kusto" to install the supported Kusto extension
"""

helps['kusto cluster create'] = """
type: command
short-summary: Create a Kusto cluster.
examples:
  - name: Create a Kusto Cluster.
    text: |-
        az kusto cluster create -l "Central US" -n myclustername -g myrgname --sku D13_v2 --capacity 10
"""

helps['kusto cluster delete'] = """
type: command
short-summary: Delete a Kusto cluster.
"""

helps['kusto cluster list'] = """
type: command
short-summary: List a Kusto cluster.
examples:
  - name: List a Kusto cluster. (autogenerated)
    text: az kusto cluster list --resource-group MyResourceGroup
    crafted: true
"""

helps['kusto cluster show'] = """
type: command
short-summary: Get a Kusto cluster.
examples:
  - name: Get a Kusto cluster. (autogenerated)
    text: az kusto cluster show --name MyCluster --resource-group MyResourceGroup
    crafted: true
"""

helps['kusto cluster start'] = """
type: command
short-summary: Start a Kusto cluster.
long-summary: When the cluster is restarted, it takes about ten minutes for it to become available (like when it was originally provisioned). It takes additional time for data to load into the hot cache.
"""

helps['kusto cluster stop'] = """
type: command
short-summary: Stop a Kusto cluster.
long-summary: When the cluster is stopped, data is not available for queries, and you can't ingest new data. Start cluster to enable queries
"""

helps['kusto cluster update'] = """
type: command
short-summary: Update a Kusto cluster.
examples:
  - name: update a Kusto Cluster.
    text: |-
        az kusto cluster update -n myclustername -g myrgname --sku D14_v2 --capacity 4
"""

helps['kusto cluster wait'] = """
type: command
short-summary: Wait for a managed Kusto cluster to reach a desired state.
long-summary: If an operation on a cluster was interrupted or was started with `--no-wait`, use this command to wait for it to complete.
"""

helps['kusto database'] = """
type: group
short-summary: Manage Azure Kusto databases.
long-summary: Azure CLI commands for Kusto are now an extension.  Execute "az extension add -n kusto" to install the extension.  The original core module for Kusto is no longer supported.
"""

helps['kusto database create'] = """
type: command
short-summary: Create a Kusto database.
examples:
  - name: create a Kusto Database.
    text: |-
        az kusto database create --cluster-name myclustername -g myrgname -n mydbname  --soft-delete-period P365D --hot-cache-period P31D
"""

helps['kusto database delete'] = """
type: command
short-summary: Delete a Kusto database.
"""

helps['kusto database list'] = """
type: command
short-summary: List a Kusto database.
examples:
  - name: List a Kusto database. (autogenerated)
    text: az kusto database list --cluster-name MyCluster --resource-group MyResourceGroup
    crafted: true
"""

helps['kusto database show'] = """
type: command
short-summary: Get a Kusto database.
examples:
  - name: Get a Kusto database. (autogenerated)
    text: az kusto database show --cluster-name MyCluster --name MyDatabase --resource-group MyResourceGroup
    crafted: true
"""

helps['kusto database update'] = """
type: command
short-summary: Update a Kusto database.
examples:
  - name: create a Kusto Database.
    text: |-
        az kusto database update --cluster-name myclustername -g myrgname -n mydbname  --soft-delete-period P365D --hot-cache-period P30D
"""

helps['kusto database wait'] = """
type: command
short-summary: Wait for a managed Kusto database to reach a desired state.
long-summary: If an operation on a database was interrupted or was started with `--no-wait`, use this command to wait for it to complete.
"""



# coding=utf-8
# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.help_files import helps  # pylint: disable=unused-import
# pylint: disable=line-too-long, too-many-lines

helps['cognitiveservices'] = """
type: group
short-summary: Manage Azure Cognitive Services accounts.
long-summary: This article lists the Azure CLI commands for Azure Cognitive Services account and subscription management only. Refer to the documentation at https://docs.microsoft.com/azure/cognitive-services/ for individual services to learn how to use the APIs and supported SDKs.
"""

helps['cognitiveservices account'] = """
type: group
short-summary: Manage Azure Cognitive Services accounts.
long-summary: This article lists the Azure CLI commands for Azure Cognitive Services account and subscription management only. Refer to the documentation at https://docs.microsoft.com/azure/cognitive-services/ for individual services to learn how to use the APIs and supported SDKs.
"""

helps['cognitiveservices account create'] = """
type: command
short-summary: Manage Azure Cognitive Services accounts.
long-summary: This article lists the Azure CLI commands for Azure Cognitive Services account and subscription management only. Refer to the documentation at https://docs.microsoft.com/azure/cognitive-services/ for individual services to learn how to use the APIs and supported SDKs.
parameters:
  - name: --kind
    populator-commands:
      - az cognitiveservices account list-kinds
  - name: --sku
    populator-commands:
      - az cognitiveservices account list-skus
examples:
  - name: Create an S0 face API Cognitive Services account in West Europe without confirmation required.
    text: az cognitiveservices account create -n myresource -g myResourceGroup --kind Face --sku S0 -l WestEurope --yes
  - name: Manage Azure Cognitive Services accounts. (autogenerated)
    text: az cognitiveservices account create --kind Face --location WestEurope --name myresource --resource-group myResourceGroup --sku S0 --subscription MySubscription --yes
    crafted: true
"""

helps['cognitiveservices account delete'] = """
type: command
short-summary: Manage Azure Cognitive Services accounts.
long-summary: This article lists the Azure CLI commands for Azure Cognitive Services account and subscription management only. Refer to the documentation at https://docs.microsoft.com/azure/cognitive-services/ for individual services to learn how to use the APIs and supported SDKs.
examples:
  - name: Delete account.
    text: az cognitiveservices account delete --name myresource-luis -g cognitive-services-resource-group
"""

helps['cognitiveservices account keys'] = """
type: group
short-summary: Manage Azure Cognitive Services accounts.
long-summary: This article lists the Azure CLI commands for Azure Cognitive Services account and subscription management only. Refer to the documentation at https://docs.microsoft.com/azure/cognitive-services/ for individual services to learn how to use the APIs and supported SDKs.
"""

helps['cognitiveservices account keys list'] = """
type: command
short-summary: Manage Azure Cognitive Services accounts.
long-summary: This article lists the Azure CLI commands for Azure Cognitive Services account and subscription management only. Refer to the documentation at https://docs.microsoft.com/azure/cognitive-services/ for individual services to learn how to use the APIs and supported SDKs.
examples:
  - name: Get current resource keys.
    text: az cognitiveservices account keys list --name myresource -g cognitive-services-resource-group
  - name: Manage Azure Cognitive Services accounts. (autogenerated)
    text: az cognitiveservices account keys list --name myresource --resource-group cognitive-services-resource-group --subscription MySubscription
    crafted: true
"""

helps['cognitiveservices account keys regenerate'] = """
type: command
short-summary: Manage Azure Cognitive Services accounts.
long-summary: This article lists the Azure CLI commands for Azure Cognitive Services account and subscription management only. Refer to the documentation at https://docs.microsoft.com/azure/cognitive-services/ for individual services to learn how to use the APIs and supported SDKs.
examples:
  - name: Get new keys for resource.
    text: az cognitiveservices account keys regenerate --name myresource -g cognitive-services-resource-group --key-name key1
"""

helps['cognitiveservices account list'] = """
type: command
short-summary: Manage Azure Cognitive Services accounts.
long-summary: This article lists the Azure CLI commands for Azure Cognitive Services account and subscription management only. Refer to the documentation at https://docs.microsoft.com/azure/cognitive-services/ for individual services to learn how to use the APIs and supported SDKs.
examples:
  - name: List all the Cognitive Services accounts in a resource group.
    text: az cognitiveservices account list -g MyResourceGroup
"""

helps['cognitiveservices account list-skus'] = """
type: command
short-summary: Manage Azure Cognitive Services accounts.
long-summary: This article lists the Azure CLI commands for Azure Cognitive Services account and subscription management only. Refer to the documentation at https://docs.microsoft.com/azure/cognitive-services/ for individual services to learn how to use the APIs and supported SDKs.
parameters:
  - name: --name -n
    long-summary: |
        --kind and --location will be ignored when --name is specified.
        --resource-group is required when when --name is specified.
  - name: --resource-group -g
    long-summary: |
        --resource-group is used when when --name is specified. In other cases it will be ignored.
  - name: --kind
    populator-commands:
      - az cognitiveservices account list-kinds
examples:
  - name: Show SKUs.
    text: az cognitiveservices account list-skus --kind Face --location westus
"""

helps['cognitiveservices account list-models'] = """
type: command
short-summary: Manage Azure Cognitive Services accounts.
long-summary: This article lists the Azure CLI commands for Azure Cognitive Services account and subscription management only. Refer to the documentation at https://docs.microsoft.com/azure/cognitive-services/ for individual services to learn how to use the APIs and supported SDKs.
examples:
  - name: List models available for a Cognitive Services account.
    text: az cognitiveservices account list-models -n myresource -g cognitive-services-resource-group
"""

helps['cognitiveservices account network-rule'] = """
type: group
short-summary: Manage network rules.
"""

helps['cognitiveservices account network-rule add'] = """
type: command
short-summary: Add a network rule.
long-summary: >
    Rules can be created for an IPv4 address, address range (CIDR format), or a virtual network subnet.
examples:
  - name: Create a rule to allow a specific address-range.
    text: az cognitiveservices account network-rule add -g myRg --name MyAccount --ip-address 23.45.1.0/24
  - name: Create a rule to allow access for a subnet.
    text: az cognitiveservices account network-rule add -g myRg --name MyAccount --vnet myvnet --subnet mysubnet
"""

helps['cognitiveservices account network-rule list'] = """
type: command
short-summary: List network rules.
examples:
  - name: List network rules.
    text: az cognitiveservices account network-rule list --name MyAccount --resource-group MyResourceGroup
    crafted: true
"""

helps['cognitiveservices account network-rule remove'] = """
type: command
short-summary: Remove a network rule.
examples:
  - name: Remove a network rule.
    text: az cognitiveservices account network-rule remove --name MyAccount --resource-group MyResourceGroup --subnet mysubnet
    crafted: true
  - name: Remove a network rule.
    text: az cognitiveservices account network-rule remove --name MyAccount --ip-address 23.45.1.0/24 --resource-group MyResourceGroup
    crafted: true
"""

helps['cognitiveservices account show'] = """
type: command
short-summary: Manage Azure Cognitive Services accounts.
long-summary: This article lists the Azure CLI commands for Azure Cognitive Services account and subscription management only. Refer to the documentation at https://docs.microsoft.com/azure/cognitive-services/ for individual services to learn how to use the APIs and supported SDKs.
examples:
  - name: Show account information.
    text: az cognitiveservices account show --name myresource --resource-group cognitive-services-resource-group
"""

helps['cognitiveservices account update'] = """
type: command
short-summary: Manage Azure Cognitive Services accounts.
long-summary: This article lists the Azure CLI commands for Azure Cognitive Services account and subscription management only. Refer to the documentation at https://docs.microsoft.com/azure/cognitive-services/ for individual services to learn how to use the APIs and supported SDKs.
parameters:
  - name: --sku
    populator-commands:
      - az cognitiveservices account list-skus
examples:
  - name: Update sku and tags.
    text: az cognitiveservices account update --name myresource -g cognitive-services-resource-group --sku S0 --tags external-app=chatbot-HR azure-web-app-bot=HR-external azure-app-service=HR-external-app-service
"""

helps['cognitiveservices list'] = """
type: command
short-summary: Manage Azure Cognitive Services accounts.
long-summary: This article lists the Azure CLI commands for Azure Cognitive Services account and subscription management only. Refer to the documentation at https://docs.microsoft.com/azure/cognitive-services/ for individual services to learn how to use the APIs and supported SDKs.
examples:
  - name: List all the Cognitive Services accounts in a resource group.
    text: az cognitiveservices list -g MyResourceGroup
"""

helps['cognitiveservices account identity'] = """
type: group
short-summary: Manage identity of Cognitive Services accounts.
"""

helps['cognitiveservices account identity assign'] = """
type: command
short-summary: Assign an identity of a Cognitive Services account.
long-summary: Assign an identity object of a Cognitive Services account. An system assigned identity will be generate and assigned to the account.
examples:
  - name: Assign an identity of Cognitive Services accounts.
    text: az cognitiveservices account identity assign --name myresource --resource-group cognitive-services-resource-group
"""

helps['cognitiveservices account identity show'] = """
type: command
short-summary: Show the identity of a Cognitive Services account.
long-summary: Show the identity object of a Cognitive Services account, empty object might be returned if the account has no assigned identity.
examples:
  - name: Show the identity of Cognitive Services accounts.
    text: az cognitiveservices account identity show --name myresource --resource-group cognitive-services-resource-group
"""

helps['cognitiveservices account identity remove'] = """
type: command
short-summary: Remove the identity from a Cognitive Services account.
long-summary: Remove the identity (if exists) from a Cognitive Services account.
examples:
  - name: Remove the identity from a Cognitive Services account.
    text: az cognitiveservices account identity remove --name myresource --resource-group cognitive-services-resource-group
"""

helps['cognitiveservices account list-deleted'] = """
type: command
short-summary: List soft-deleted Azure Cognitive Services accounts.
long-summary: This article lists the Azure CLI commands for Azure Cognitive Services account and subscription management only. Refer to the documentation at https://docs.microsoft.com/azure/cognitive-services/ for individual services to learn how to use the APIs and supported SDKs.
examples:
  - name: List all the Cognitive Services accounts in a subscription.
    text: az cognitiveservices account list-deleted
"""


helps['cognitiveservices account show-deleted'] = """
type: command
short-summary: Show a soft-deleted Azure Cognitive Services account.
long-summary: This article lists the Azure CLI commands for Azure Cognitive Services account and subscription management only. Refer to the documentation at https://docs.microsoft.com/azure/cognitive-services/ for individual services to learn how to use the APIs and supported SDKs.
examples:
  - name: Show a soft-deleted Azure Cognitive Services account.
    text: az cognitiveservices account show-deleted --location eastus --resource-group cognitive-services-resource-group --name cognitive-services-account-name
"""

helps['cognitiveservices account recover'] = """
type: command
short-summary: Recover a soft-deleted Azure Cognitive Services account.
long-summary: This article lists the Azure CLI commands for Azure Cognitive Services account and subscription management only. Refer to the documentation at https://docs.microsoft.com/azure/cognitive-services/ for individual services to learn how to use the APIs and supported SDKs.
examples:
  - name: Recover a soft-deleted Azure Cognitive Services account.
    text: az cognitiveservices account recover --location eastus --resource-group cognitive-services-resource-group --name cognitive-services-account-name
"""

helps['cognitiveservices account purge'] = """
type: command
short-summary: Purge a soft-deleted Azure Cognitive Services account.
long-summary: This article lists the Azure CLI commands for Azure Cognitive Services account and subscription management only. Refer to the documentation at https://docs.microsoft.com/azure/cognitive-services/ for individual services to learn how to use the APIs and supported SDKs.
examples:
  - name: Purge a soft-deleted Azure Cognitive Services account.
    text: az cognitiveservices account purge --location eastus --resource-group cognitive-services-resource-group --name cognitive-services-account-name
"""

helps['cognitiveservices account deployment'] = """
type: group
short-summary: Manage deployments for Azure Cognitive Services accounts.
"""

helps['cognitiveservices account deployment create'] = """
type: command
short-summary: Create a deployment for Azure Cognitive Services account.
long-summary: This article lists the Azure CLI commands for Azure Cognitive Services account and subscription management only. Refer to the documentation at https://docs.microsoft.com/azure/cognitive-services/ for individual services to learn how to use the APIs and supported SDKs.
examples:
  - name: Create a deployment for Azure Cognitive Services account.
    text: az cognitiveservices account deployment create -g yuanyang-test-sdk -n yytest-oai --deployment-name dpy --model-name ada --model-version "1" --model-format OpenAI --scale-settings-capacity 1 --scale-settings-scale-type "Manual"
"""

helps['cognitiveservices account deployment delete'] = """
type: command
short-summary: Delete a deployment from Azure Cognitive Services account.
long-summary: This article lists the Azure CLI commands for Azure Cognitive Services account and subscription management only. Refer to the documentation at https://docs.microsoft.com/azure/cognitive-services/ for individual services to learn how to use the APIs and supported SDKs.
examples:
  - name: Delete a deployment from Azure Cognitive Services account.
    text: az cognitiveservices account deployment delete -g yuanyang-test-sdk -n yytest-oai --deployment-name dpy
"""

helps['cognitiveservices account deployment show'] = """
type: command
short-summary: Show a deployment for Azure Cognitive Services account.
long-summary: This article lists the Azure CLI commands for Azure Cognitive Services account and subscription management only. Refer to the documentation at https://docs.microsoft.com/azure/cognitive-services/ for individual services to learn how to use the APIs and supported SDKs.
examples:
  - name: Show a deployment for Azure Cognitive Services account.
    text: az cognitiveservices account deployment show -g yuanyang-test-sdk -n yytest-oai --deployment-name dpy
"""

helps['cognitiveservices account deployment list'] = """
type: command
short-summary: Show all deployments for Azure Cognitive Services account.
long-summary: This article lists the Azure CLI commands for Azure Cognitive Services account and subscription management only. Refer to the documentation at https://docs.microsoft.com/azure/cognitive-services/ for individual services to learn how to use the APIs and supported SDKs.
examples:
  - name: Show all deployments for Azure Cognitive Services account.
    text: az cognitiveservices account deployment list -g yuanyang-test-sdk -n yytest-oai
"""

helps['cognitiveservices commitment-tier'] = """
type: group
short-summary: Manage commitment tiers for Azure Cognitive Services.
"""

helps['cognitiveservices commitment-tier list'] = """
type: command
short-summary: Show all commitment tiers for Azure Cognitive Services.
long-summary: This article lists the Azure CLI commands for Azure Cognitive Services account and subscription management only. Refer to the documentation at https://docs.microsoft.com/azure/cognitive-services/ for individual services to learn how to use the APIs and supported SDKs.
examples:
  - name: Show all commitment tiers for Azure Cognitive Services.
    text: az cognitiveservices commitment-tier list -l centraluseuap
"""

helps['cognitiveservices account commitment-plan'] = """
type: group
short-summary: Manage commitment plans for Azure Cognitive Services accounts.
"""

helps['cognitiveservices account commitment-plan create'] = """
type: command
short-summary: Create a commitment plan for Azure Cognitive Services account.
long-summary: This article lists the Azure CLI commands for Azure Cognitive Services account and subscription management only. Refer to the documentation at https://docs.microsoft.com/azure/cognitive-services/ for individual services to learn how to use the APIs and supported SDKs.
examples:
  - name: Create a commitment plan for Azure Cognitive Services account.
    text: az cognitiveservices account commitment-plan create -g yuanyang-test-sdk -n yytest-ta --commitment-plan-name "plan" --hosting-model "Web" --plan-type "TA" --auto-renew false --current-tier "T1" --next-tier "T2"
"""

helps['cognitiveservices account commitment-plan delete'] = """
type: command
short-summary: Delete a commitment plan from Azure Cognitive Services account.
long-summary: This article lists the Azure CLI commands for Azure Cognitive Services account and subscription management only. Refer to the documentation at https://docs.microsoft.com/azure/cognitive-services/ for individual services to learn how to use the APIs and supported SDKs.
examples:
  - name: Delete a commitment plan from Azure Cognitive Services account.
    text: az cognitiveservices account commitment-plan delete -g yuanyang-test-sdk -n yytest-ta --commitment-plan-name "plan"
"""

helps['cognitiveservices account commitment-plan show'] = """
type: command
short-summary: Show a commitment plan from Azure Cognitive Services account.
long-summary: This article lists the Azure CLI commands for Azure Cognitive Services account and subscription management only. Refer to the documentation at https://docs.microsoft.com/azure/cognitive-services/ for individual services to learn how to use the APIs and supported SDKs.
examples:
  - name: Show a commitment plan from Azure Cognitive Services account.
    text: az cognitiveservices account commitment-plan show -g yuanyang-test-sdk -n yytest-ta --commitment-plan-name "plan"
"""

helps['cognitiveservices account commitment-plan list'] = """
type: command
short-summary: Show all commitment plans from Azure Cognitive Services account.
long-summary: This article lists the Azure CLI commands for Azure Cognitive Services account and subscription management only. Refer to the documentation at https://docs.microsoft.com/azure/cognitive-services/ for individual services to learn how to use the APIs and supported SDKs.
examples:
  - name: Show all commitment plans from Azure Cognitive Services account.
    text: az cognitiveservices account commitment-plan list -g yuanyang-test-sdk -n yytest-ta
"""
